#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: SystemBackup-lib.pl,v 1.2 2002/05/13 19:29:39 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  Load the WebTool library, the flexbackup config, and do init_config().
  do '/usr/webtool/web-lib.pl';
  require "/etc/flexbackup.conf";
  &init_config();


  ### ##########################################################################
  ### Global Variables
  ### ##########################################################################
  $FLEXBACKUP_BIN     =  '/usr/bin/flexbackup';
  $FLEXBACKUP_LOGDIR  =  '/var/log/flexbackup';

  %BACKUP_TYPES_SHORT =  ( '0',    'Full',
                           '9',    'Incremental' );

  %BACKUP_TYPES       =  ( '0',    'Full Backup',
                           '9',    'Incremental Backup' );

  %BACKUP_TIMES       =  ( 'NEVER',    'Never',
                           'daily',    'Every Day',
                           'weekly',   'Every Week',
                           'monthly',  'Every Month' );

  %BACKUP_DEVICES     =  ( '/var/BACKUP',  'Backup to File',
                           '/dev/nst0',    'Backup to SCSI Tape Drive',
                           '/dev/nht0',    'Backup to IDE Tape Drive' );

  %TRUEFALSE          = ( 'true', 'Yes',
                          'false', 'No' );


  ### ##########################################################################
  sub ListNamedBackups() {
    my ($line, %BACKUPS);

    $line = 0;

    open CONFIG, "$module_config_directory/config";
    while (<CONFIG>)
    {
      $line++;
      chomp;

      local ($key, $value)    =  split /\=/, $_;
      local ($bname, $btype)  =  split /\_/, $key;

      ###  Get out if this isn't a backupNUM_ line.
      next  if ($bname !~ m/^backup[0-9]+$/);

      ###  Set the key/value.
      $BACKUPS{$bname}->{$btype} = $value;

      ###  Set our line number, if not already set.
      if ($BACKUPS{$bname}->{'name'} && !$BACKUPS{$bname}->{'lineno'}) {
        $BACKUPS{$bname}->{'lineno'}  = $line;
      }
    }
    close CONFIG;

    return %BACKUPS;
  }


  ### ##########################################################################
  sub ListStoredBackups() {
    my ($b, %backups, @BACKUPS);

    ($b)      =  @_;
    %backups  =  &ListNamedBackups();

    ###  Case 1:  We are using file backups, so read the directory.
    if ($device eq "/var/BACKUP") {
      local ($base, @files, $level, $date);

      $base   =  $backups{$b}->{'directory'};
      $base   =~ s/\//\-/g;
      $base   =~ s/^\-//;
      @files  =  glob "/var/BACKUP/$base\.*";

      foreach (@files) {
        my ($file, $level, $date, $date2);
        $file            = $_;
        ($level, $date)  = ($file =~ m/\/$base\.([0-9]+)\.([0-9]+)\.afio\-gz$/);
        $date2           =  &FormatDate($date);

        push @BACKUPS, { 'level' => $level, 'date' => $date2,
                         'file' => $file,   'logstamp' => $date };
      }

      return @BACKUPS;
    }

    ###  Case 2:  We are using tape backups, so read the statedir.
    else {
      local ($base, @files, @tmp);

      $base   =  $backups{$b}->{'directory'};
      $base   =~ s/\//\-/g;
      $base   =~ s/^\-//;
      @files  =  glob "/var/state/flexbackup/$base\.*";

      foreach my $f (@files) {
        local ($date, $yr, $mo, $da, $level);

        $date     =  scalar localtime((stat($f))[9]);
        $yr       =  ((localtime((stat($f))[9]))[5]) + 1900;
        $mo       =  ((localtime((stat($f))[9]))[4]) + 1;
        $da       =  ((localtime((stat($f))[9]))[3]);
        $da       =  "0$da" if ($da < 10);
        $mo       =  "0$mo" if ($mo < 10);
        ($level)  =  ($f =~ m/\.([0-9])$/);

        push @BACKUPS, { 'level' => $level, 'date'      => $date,
                         'file'  => "TAPE", 'datestamp' => "$yr$mo$da",
                         'base'  => $base,  'logstamp'  => "$yr$mo$da" };
      }

      return @BACKUPS;
    }

    ###  ===>  NOT REACHED  <==  ###
  }


  ### ##########################################################################
  sub FormatDate() {
    my (%MONTHS, $year, $month, $day, $rv);

    %MONTHS  =  ( '01' => 'January', '02' => 'February', '03' => 'March',
                  '04' => 'April',   '05' => 'May',      '06' => 'June',
                  '07' => 'July',    '08' => 'August',   '09' => 'September',
                  '10' => 'October', '11' => 'November', '12' => 'December' );

    ($year, $month, $day)  =  ($_[0] =~ m/^([0-9]{4})([0-9]{2})([0-9]{2})$/);
    $rv                    = "$MONTHS{$month} $day, $year";

    return $rv;
  }


  ### ##########################################################################
  sub GetNextBackupName() {
    my ($rv, %b, $last_key);
    %b  =  &ListNamedBackups();

    ###  Get our $last_key.
    foreach $k (sort keys %b) { $last_key = $k; }

    ###  Now get the last key's ID, then increment it.
    ($rv) = ($last_key =~ m/^backup([0-9]+)$/); $rv++;

    return "backup$rv";
  }


  ### ##########################################################################
  sub GetStartBlock() {

    open LOG, "gzip -dc $FLEXBACKUP_LOGDIR/$_[0].$_[2].$_[1].gz |";
    while (my $line = <LOG>)
    {
      chomp $line;

      if ($line =~ m/^At\ block\ ([0-9]+)\.$/) { return $1; }
    }
    close LOG;

    return undef;
  }


  ### ##########################################################################
  sub PrintError {
    my ($message);
    ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>
ErrorEOF
  }

  ### ##########################################################################
  1;
